include "m8c.inc"
include "PWM16_1.inc"
include "DAC8_1.inc"
include "macros.inc"

export Initialize
export ScanKeyBoard
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initalize
;;
;; Get every thing started.
;;
;;  No Inputs
;;  Returns Nothing
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
area text(ROM,REL)
Initialize:
   mov reg[PRT2DR],00h         ;Get the keyboard ready
   mov reg[PRT1DR], 00h
   mov [cInstrument],(>PianoNote)
   mov [cTaper],(>PianoTaper3)
   mov [cNextTone],0
   mov X,4
   repeat1:
      mov [X + fKeyStatus],0
      mov [X + fOldReadKeys],0
      mov [X + fReadKeys],0
      dec X
   until1: jnc repeat1
   mov X,MAXVOICES
   repeat2:
      mov[X + cVoiceStatus],255
      dec X
   until2: jnc repeat2
   mov [cKeyTimer],cKeyTimerLoad
   mov [fProcessKeys],0

   mov A, DAC8_1_HIGHPOWER 
   call DAC8_1_Start
   mov A,0
   call DAC8_1_WriteStall
   
   call PWM16_1_EnableInt
   call PWM16_1_Start
 ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  ScanKeyboard
;;
;;  This routine reads the key port
;;     Determines which keys are to be turned off or on
;;     Process the aproppiate voice
;;  Inputs  A contains Scan value
;;  Ouptus None
;;  Registers  cTemp is use to store key value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
export fTurnOn
export fTurnOff
export cTemp
area bss(RAM) 
   fTurnOff:        blk 1 ;
   fTurnOn:         blk 1 ;
   cTemp:           blk 1 ;
   cKeyWalk:        blk 1 ;
area text(ROM,REL)

ScanKeyBoard:
   mov X,A                    ;use the correct key read registers
   asl A
   asl A
   asl A
   mov [cTemp],A               ;Save the base key value  (8* X)
   
   mov A,[X + fReadKeys]
   mov [X + fOldReadKeys],A
   mov A,reg[PRT1DR]
;;;;   cpl A
   mov [X + fReadKeys],A      ;ReadKey now has new values

   mov A,[X + fOldReadKeys]
   or  A,[X +fReadKeys]
   cpl A
   and A,[X + fKeyStatus]
   mov [fTurnOff],A           ;Voices to be turned Off
      
   mov A,[ X + fKeyStatus]
   cpl A
   and A,[X + fOldReadKeys]
   and A,[X + fReadKeys]
   mov [fTurnOn],A            ;Voices to be turned on
      
   mov A,[X + fOldReadKeys]
   or  A,[X + fReadKeys]
   and A,[X + fKeyStatus]
   or  A,[fTurnOn]
   mov [X + fKeyStatus],A      ;New Key Status
   ;Note X is now available
   
   ;Check for Voices to turn off
   mov [cKeyWalk],01h
   loop3:
      mov A,[fTurnOff]
      and A,[cKeyWalk]
      jz endif4
      if4:;(to turn off)
         mov A,[cTemp]
         call OffVoice
      endif4:
      asl [cKeyWalk]
      exitif3: jz endloop3
      inc [cTemp]
      jmp loop3
   endloop3:
   sub [cTemp],7
   
   ;Check for Voices to turn on
   mov [cKeyWalk],01h
   loop4:
      mov A,[fTurnOn]
      and A,[cKeyWalk]
      jz endif5
      if5:;(to turnon)
         mov A,[cTemp]
         call cVoiceAvailable
         cmp A,255
         jnz else6
         if6:;(no voice available)
            mov A,[cTemp]
            asr A
            asr A
            asr A
            mov X,A
            mov A,[cKeyWalk]
            cpl A
            and [X + fKeyStatus],A   ;check key next time
            jmp endif6
         else6:;(voice available)
            mov X,A
            mov A,[cTemp]
            call StartVoice
            mov A,[cTemp]
            mov [X + cVoiceStatus],A
         endif6:
      endif5:
      
      asl [cKeyWalk]
      exitif4: jz endloop4
      inc [cTemp]
      jmp loop4
   endloop4:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  OffVoice
;;
;;  This rountine checks all the voice to see if it's
;;  voice is the same has the one stored in "A".
;;  If so the release set set for that voices taper
;;
;;  Inputs  Key value in A
;;  Outputs None
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
OffVoice:
   mov X,MAXVOICES
   repeat3:
      cmp A,[X + cVoiceStatus]
      jnz endif1
      if1:;(voice is a match)
         mov[X + cVoiceTaperDel],Release
      endif1:   
      dec X
   until3: jnc repeat3
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  cVoiceAvailable
;;  If if a Voice is avaible.
;;  If so the Voice Value is Returned in A
;;  If not 255 is return in "A"
;;  Inputs   none
;;  Outputs  available voice is returned in "A"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
cVoiceAvailable:
   mov X,MAXVOICES
   repeat4:
      cmp [X + cVoiceStatus],255
      jnz endif2
      if2:;(Voice is available)
         mov A,X
         ret
      endif2:
      dec X
   until4: jnc repeat4
   mov A,255
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Start_Voice
;; Setup the voice to be used
;; The voice to be seleted is in X
;; The note is stored in A
;; Inputs:
;;      A  the note to be played
;;      X  which voice is to be used
;; Returns: None
;;
;; Registers affected
;;      iVoiceMS(X)
;;      iVoiceLS(X)
;;      iVoiceDeltaMS(X)
;;      iVoiceDeltaLS(X)    
;;      iVoiceTaperMS(X)
;;      iVoiceTaperLS(X)     
;;      cVoiceTaperDel(X)
;;      cVoiceStatus(X)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
export StartVoice
StartVoice:
   mov [X + iVoiceMS],0              ;set for round off
   mov [X + iVoiceLS],80h
   mov [X + iVoiceTaperMS],0
   mov [X + iVoiceTaperLS],0
   mov [X + cVoiceTaperDel],Sustain
   asl A     
   push A                             ;Load Delta
   index Notes
   mov [X + iVoiceDeltaMS],A
   pop A
   index (Notes + 1 )
   mov [X + iVoiceDeltaLS],A
ret   
MainFreq: equ 15000* 10;
Sustain: equ (((65536*8) +(MainFreq/2))/MainFreq)
export Release
Release:  equ (((65536*80)+(MainFreq/2))/MainFreq)  
Notes:  ;40 ntoes starting with A 110
   dw (((65536*1100)+(MainFreq/2))/MainFreq) ; A
   dw (((65536*1165)+(MainFreq/2))/MainFreq) ;
   dw (((65536*1235)+(MainFreq/2))/MainFreq) ; B
   dw (((65536*1308)+(MainFreq/2))/MainFreq) ; C
   dw (((65536*1386)+(MainFreq/2))/MainFreq) ;
   dw (((65536*1468)+(MainFreq/2))/MainFreq) ; D
   dw (((65536*1556)+(MainFreq/2))/MainFreq) ;
   dw (((65536*1648)+(MainFreq/2))/MainFreq) ; E
   dw (((65536*1746)+(MainFreq/2))/MainFreq) ; F
   dw (((65536*1850)+(MainFreq/2))/MainFreq) ;
   dw (((65536*1960)+(MainFreq/2))/MainFreq) ; G
   dw (((65536*2077)+(MainFreq/2))/MainFreq) ;
   dw (((65536*2200)+(MainFreq/2))/MainFreq) ; A
   dw (((65536*2331)+(MainFreq/2))/MainFreq) ; 
   dw (((65536*2469)+(MainFreq/2))/MainFreq) ; B
   dw (((65536*2616)+(MainFreq/2))/MainFreq) ; C
   dw (((65536*2772)+(MainFreq/2))/MainFreq) ;
   dw (((65536*2937)+(MainFreq/2))/MainFreq) ; D
   dw (((65536*3111)+(MainFreq/2))/MainFreq) ;
   dw (((65536*3296)+(MainFreq/2))/MainFreq) ; E
   dw (((65536*3492)+(MainFreq/2))/MainFreq) ; F
   dw (((65536*3700)+(MainFreq/2))/MainFreq) ;
   dw (((65536*3920)+(MainFreq/2))/MainFreq) ; G
   dw (((65536*4153)+(MainFreq/2))/MainFreq) ;
   dw (((65536*4400)+(MainFreq/2))/MainFreq) ; A
   dw (((65536*4662)+(MainFreq/2))/MainFreq) ;
   dw (((65536*4939)+(MainFreq/2))/MainFreq) ; B
   dw (((65536*5233)+(MainFreq/2))/MainFreq) ; C
   dw (((65536*5544)+(MainFreq/2))/MainFreq) ;
   dw (((65536*5873)+(MainFreq/2))/MainFreq) ; D
   dw (((65536*6223)+(MainFreq/2))/MainFreq) ; 
   dw (((65536*6593)+(MainFreq/2))/MainFreq) ; E
   dw (((65536*6985)+(MainFreq/2))/MainFreq) ; F
   dw (((65536*7400)+(MainFreq/2))/MainFreq) ;
   dw (((65536*7840)+(MainFreq/2))/MainFreq) ; G
   dw (((65536*8306)+(MainFreq/2))/MainFreq) ;   
   dw (((65536*8800)+(MainFreq/2))/MainFreq) ; A
   dw (((65536*9323)+(MainFreq/2))/MainFreq) ; 
   dw (((65536*9878)+(MainFreq/2))/MainFreq) ; B
   dw (((65536*10465)+(MainFreq/2))/MainFreq); C      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




