;============================================================================
;  FILENAME:   boot.asm
;   VERSION:   2.1
;      DATE:   10 October 2001
;
;  DESCRIPTION:
;     M8C Boot Code from Reset.
;
;   Copyright (c) Cypress MicroSystems 2001. All rights reserved.
;
; NOTE:
;
; PSoC Designer's Device Editor uses a template file, BOOT.TPL, from the
; project's root directory to create BOOT.ASM.  Changes made to Boot.asm
; get overwritten every time the code gets regenerated; consequently such
; changes should be made to BOOT.TPL instead. Care must be taken when
; modifying BOOT.TPL so that the replacement strings in the interrupt
; table do not become corrupted.
;============================================================================

include ".\lib\`@PROJECT_NAME`_GlobalParams.inc"
include ".\lib\m8c.inc"

;-----------------------------------------------
; Optmization flags
;-----------------------------------------------

C_LANGUAGE_SUPPORT: equ 0       ; Set to 0 to optimize for ASM only
OPTIMIZE_FOR_SPACE: equ 1       ; Currently unsupported

;-----------------------------------------------
; Export Declarations
;-----------------------------------------------

export __start
export _exit
export __bss_start

IF C_LANGUAGE_SUPPORT
export __lit_start
export __idata_start
export __data_start
ENDIF

;-----------------------------------------------
; Interrupt Vector Table
;-----------------------------------------------
;
; Interrupt vector table entries are 4 bytes long and contain the code
; that services the interrupt (or causes it to be serviced).
;
;-----------------------------------------------


    AREA    TOP(ROM, ABS)

    org 0                       ; Reset Interrupt Vector
    jmp __start                 ; First instruction executed following a Reset

    org 04h                     ; Supply Monitor Interrupt Vector
    `@INTERRUPT_1`
    reti

    org 08h                     ; PSoC Block DBA00 Interrupt Vector
    `@INTERRUPT_2`
    reti

    org 0Ch                     ; PSoC Block DBA01 Interrupt Vector
    `@INTERRUPT_3`
    reti

    org 10h                     ; PSoC Block DBA02 Interrupt Vector
    `@INTERRUPT_4`
    reti

    org 14h                     ; PSoC Block DBA03 Interrupt Vector
    `@INTERRUPT_5`
    reti

    org 18h                     ; PSoC Block DCA04 Interrupt Vector
    `@INTERRUPT_6`
    reti

    org 1Ch                     ; PSoC Block DCA05 Interrupt Vector
    `@INTERRUPT_7`
    reti

    org 20h                     ; PSoC Block DCA06 Interrupt Vector
    `@INTERRUPT_8`
    reti

    org 24h                     ; PSoC Block DCA07 Interrupt Vector
    `@INTERRUPT_9`
    reti

    org 28h                     ; Analog Column 0 Interrupt Vector
    `@INTERRUPT_10`
    reti

    org 2Ch                     ; Analog Column 1 Interrupt Vector
    `@INTERRUPT_11`
    reti

    org 30h                     ; Analog Column 2 Interrupt Vector
    `@INTERRUPT_12`
    reti

    org 34h                     ; Analog Column 3 Interrupt Vector
    `@INTERRUPT_13`
    reti

    org 38h                     ; GPIO Interrupt Vector
    `@INTERRUPT_14`
    reti

    org 3Ch                     ; Sleep Timer Interrupt Vector
    jmp SleepTimerISR
    reti

;---------------------
;  Sleep Timer ISR
;---------------------
SleepTimerISR:
    push A
    mov A, PLL_MODE
    jz  NormalSleep
    M8C_SetBank1
    tst reg[OSC_CR0], OSC_CR0_PLL_Mode
    jnz AlreadyOn               ; if it's already set, then normal sleep
    mov A, reg[OSC_CR0]         ; Read the Oscillator control register
    and A, ~OSC_CR0_Sleep       ; Clear sleep timer frequency bits
    or  A, (PLL_MODE_JUST | SLEEP_TIMER_JUST)
                                ; Enable the PLL and set the sleep timer as
                                ;   set through parameterization
    mov reg[OSC_CR0], A
    M8C_SetBank0
    and reg[INT_MSK0], ~INT_MSK0_Sleep
                                ; mask the Sleep Timer interrupt
AlreadyOn:
    M8C_SetBank0

NormalSleep:
    //  insert your sleep code here (in boot.tpl)
    pop A
    reti


;----------------------
;  Start of Execution
;----------------------
__start:
    mov A,__bss_end             ; Set top of stack to end of used RAM
    swap SP,A
    ;-----------------------------------------------
    ; Initialize oscillator register
    ;-----------------------------------------------
    M8C_SetBank1
    mov A, PLL_MODE             ; Check for PLL Mode enabled
    jnz PLLOn
    mov reg[OSC_CR0], (CPU_CLOCK_JUST | SELECT_32K_JUST | SLEEP_TIMER_JUST)
    M8C_SetBank0
    jmp EndPLLInit
PLLOn:
    mov reg[E0h], (CPU_CLOCK_JUST | SELECT_32K_JUST | 18h)
                                ; set sleep timer for 1 second for now
    M8C_SetBank0
    or  reg[INT_MSK0], INT_MSK0_Sleep
    mov reg[RES_WDT], 38h       ; reset the sleep timer
EndPLLInit:

    ; default CT block RTopMux to OUT and RBotMux to AGND
    mov reg[ACA00CR0], 05h
    mov reg[ACA01CR0], 05h
    mov reg[ACA02CR0], 05h
    mov reg[ACA03CR0], 05h
    lcall LoadConfigInit        ; Configure PSoC blocks per Dev Editor

    IF C_LANGUAGE_SUPPORT
        call InitCRunTime       ; Initialize for C language
    ENDIF

    mov A, PLL_MODE             ; Check for PLL Mode enabled
    jz  NoPLL                   ; If not enabled, then don't enable global interrupts
    M8C_EnableGInt              ; Enable global interrupts
NoPLL:
    lcall _main                 ; Call main with A = 1
_exit:
    jmp _exit

IF C_LANGUAGE_SUPPORT
;----------------------------------------------
;;    C Runtime Environment Initialization
;----------------------------------------------

InitCRunTime:
    ;-------------------
    ; clear bss segment
    ;-------------------
    mov A,0
    mov [__r0],<__bss_start
BssLoop:
    cmp [__r0],<__bss_end
    jz BssDone
    mvi [__r0],A
    jmp BssLoop
BssDone:
    ;----------------------------
    ; copy idata to data segment
    ;----------------------------
    mov A,>__idata_start
    mov X,<__idata_start
    mov [__r0],<__data_start
IDataLoop:
    cmp [__r0],<__data_end
    jz IDataDone
    push A
    romx
    mvi [__r0],A
    pop A
    inc X
    adc A,0
    jmp IDataLoop
IDataDone:
    ret

;------------------------------------------------------
;;  RAM segments for C CONST, static & global items
;------------------------------------------------------
ENDIF

    AREA lit
__lit_start:

    AREA idata
__idata_start:

    AREA data(ram)
__data_start:




    AREA virtual_registers(ram)
__virtual_registers_end:

;---------------------------------------------
;         RAM segment for general use
;---------------------------------------------
    AREA bss(ram)
__bss_start:


; end of file
