;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Project for Application Note AN2024
;
;   Copyright (c) 2001, 2002 Cypress MicroSystems, Inc. All rights reserved.
;
;      Analog Outputs  Port0.3 & Port0.7
;      Keyboard stuinmulus Port2.0-4
;      Keyboard input Port1.0-7
;  Note: The the hardware is set up to drive a 5 by 8 X-Y keyboard.
;  The switches must have diodes to allow multiple key presses with
;  no ghost keys.  The diodes are connected with the anodes to Port 2
;  and the cathodes connected to port 1.  If your key is reverse of this
;  please contact Cypress MicroSystems Applications Department.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

export _main
area text(ROM,REL)
include "m8c.inc"
include "PWM16_1.inc"
include "DAC8_1.inc"
include "macros.inc"

export iVoiceMS
export iVoiceLS
export iVoiceDeltaMS
export iVoiceDeltaLS    
export iVoiceTaperMS
export iVoiceTaperLS     
export cVoiceTaperDel
export cVoiceStatus
export fKeyStatus
export fOldReadKeys
export fReadKeys   
export fProcessKeys
export cTaper
export cInstrument
export cKeyTimer
export cNextTone
export cKeyTimerLoad

MAXVOICES: equ 9  ;0 through 9
export MAXVOICES
area bss(RAM) 
   iVoiceMS:        blk (MAXVOICES + 1)
   iVoiceLS:        blk (MAXVOICES + 1)
   iVoiceDeltaMS:   blk (MAXVOICES + 1)
   iVoiceDeltaLS:   blk (MAXVOICES + 1)    
   iVoiceTaperMS:   blk (MAXVOICES + 1)
   iVoiceTaperLS:   blk (MAXVOICES + 1)     
   cVoiceTaperDel:  blk (MAXVOICES + 1)
   cVoiceStatus:    blk (MAXVOICES + 1)
   fKeyStatus:      blk 5 ;The state of the 8 keys
   fOldReadKeys:    blk 5 ;
   fReadKeys:       blk 5 ;   
   cNextTone:       blk 1 ;The value set to the DAC
   cInstrument:     blk 1 ;The instrument to be played
   cTaper:          blk 1 ;The taper for the selected instrument
   cKeyTimer:       blk 1 ;
   fProcessKeys:    blk 1 ;Flag that keys should be examined
area text(ROM,REL)

   cKeyTimerLoad:   equ  256;
_main:
   mov reg[84h],adh		;SCBLOCK_1_SCBLOCKcr0
   mov reg[85h],20h		;SCBLOCK_1_SCBLOCKcr1
   mov reg[86h],a0h		;SCBLOCK_1_SCBLOCKcr2
   mov reg[87h],33h		;SCBLOCK_1_SCBLOCKcr3
   mov reg[80h],30h		;SCBLOCK_2_SCBLOCKcr0
   mov reg[81h],20h		;SCBLOCK_2_SCBLOCKcr1
   mov reg[82h],a0h		;SCBLOCK_2_SCBLOCKcr2
   mov reg[83h],33h		;SCBLOCK_2_SCBLOCKcr3
  ;Initiize
   M8C_EnableGInt
   call Initialize
   loop1:
      wait1:;(wait for keyboard timer)
         cmp [fProcessKeys],0
      jz wait1
      mov [fProcessKeys],0
      mov reg[PRT2DR],01h
      mov A,0               ;Scan 0
      call ScanKeyBoard
      
      mov reg[PRT2DR],02h
      mov A,1               ;Scan 1
      call ScanKeyBoard
     
      mov reg[PRT2DR],04h
      mov A,2               ;Scan 2
      call ScanKeyBoard
     
      mov reg[PRT2DR],08h
      mov A,3               ;Scan 3
      call ScanKeyBoard
     
      mov reg[PRT2DR],10h
      mov A,4               ;Scan 4
      call ScanKeyBoard
      
   endloop1: jmp loop1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Voice Interrupt
;;
;;  This is where the 8 voices are generated 
;;  and outputed to the DAC
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
export   PWM16_1INT
PWM16_1INT:
VoiceInterrupt:
   push A 
   push X 
 
   mov  A,[cNextTone]         ;Output the last tone
   index LScr0
   push A
   mov  A,[cNextTone]
   index MScr0
   M8C_Stall
   mov  reg[DAC8_1_MSB_CR0], A
   M8C_Unstall
   pop A
   mov reg[DAC8_1_LSB_CR0],A

   ProcessVoice 0
   ProcessVoice 1                                                
   ProcessVoice 2
   ProcessVoice 3
   ProcessVoice 4
   ProcessVoice 5
   ProcessVoice 6
   ProcessVoice 7
   ProcessVoice 8
   ProcessVoice 9
;-------------------------------------------------------------------
   mov reg[MAC_CL0],0     ;clear MAC                        
   mov reg[MUL_X],40h     ;insert round off value
   mov reg[MAC_Y],02h
;--------------------------
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack
   MACVoiceFromStack   
   MACVoiceFromStack
   MACVoiceFromStack
;--------------------------
   mov A,reg[ACC_DR2]
   jz  else1
   cpl A
   jz  else1
   if1:;(out of range)
      and A,80h   ; 0 is neg  80 if pos
      jnz else2
      if2:;(pos)
         mov A,7fh
         jmp endif2
      else2:;(neg)
         mov A,80h
      endif2:
      jmp endif1
   else1:;(value is in range)
      mov A,reg[ACC_DR1]
   endif1:
   mov [cNextTone],A
   pop X
   pop A
 
   dec [cKeyTimer]
   jnc endif3
   if3:;(Counter has counter down
      mov [fProcessKeys],1
      mov [cKeyTimer],cKeyTimerLoad
   endif3:
   reti
include "DACData.inc"
